/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.exception;


import ws.ladder.model.ExtendedDataObject;
import ws.ladder.model.Property;
import ws.ladder.util.I18NUtils;

public class BidirOutOfSyncException extends RuntimeException {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private ExtendedDataObject object;
	private Property property;
	private ExtendedDataObject otherObject;
	private Property otherProperty;	
	
	public BidirOutOfSyncException(ExtendedDataObject object, Property property, ExtendedDataObject otherObject, Property otherProperty) {
		this.object = object;
		this.property = property;
		this.otherObject = otherObject;
		this.otherProperty = otherProperty;
	}

	public Property getOtherProperty() {
		return otherProperty;
	}

	public void setOtherProperty(Property otherProperty) {
		this.otherProperty = otherProperty;
	}
	
	public ExtendedDataObject getObject() {
		return object;
	}

	public void setObject(ExtendedDataObject object) {
		this.object = object;
	}

	public Property getProperty() {
		return property;
	}

	public void setProperty(Property property) {
		this.property = property;
	}

	public ExtendedDataObject getOtherObject() {
		return otherObject;
	}

	public void setOtherObject(ExtendedDataObject otherObject) {
		this.otherObject = otherObject;
	}	
	
	@Override
	public String getMessage() {
		String[] params = new String[4];
		params[0] = object.getType().getName() + "(id: " + object.getIdentifierValue() + ")";
		params[1] = property.getName();
		params[2] = otherObject.getType().getName() + "(id: " + otherObject.getIdentifierValue() + ")";
		params[3] = otherProperty.getName();		
		return  I18NUtils.getResource(  "exception.bidirOutOfSync",I18NUtils.CORE_RESOURCES, params);			
	}
}
