/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.exception;

import java.util.HashSet;
import java.util.Set;

import ws.ladder.model.Type;
import ws.ladder.util.I18NUtils;

public class MultipleClassForPropertyException extends RuntimeException {

	private static final long serialVersionUID = 1L;
	
	private Type entityType;
	private String propertyName;
	private Set<Class<?>> matchedClasses;

	public Set<Class<?>> getMatchedClasses() {
		return matchedClasses;
	}

	public void setMatchedClasses(Set<Class<?>> matchedClasses) {
		this.matchedClasses = matchedClasses;
	}

	public MultipleClassForPropertyException(Type entityType, String propertyName, Object matchedClasses) {
		// Multiple properties in the subclass for the entityType matched the property
		// This happened because of the static (default) type narrowing strategy.
		// A custom type narrower will have to be provided to solve this.
		
		this.entityType = entityType;
		this.propertyName = propertyName;
		
		if(Set.class.isAssignableFrom(matchedClasses.getClass())) {
			this.matchedClasses = new HashSet<Class<?>>();
			
			Set<Class<?>> classes = (Set<Class<?>>) matchedClasses;
			for(Class<?> clazz: classes)
				this.matchedClasses.add(clazz);
		}
	}

	@Override
	public String getMessage(){
		String[] params = new String[3];
		params[0] = entityType.getInstanceClass().getName();
		params[1] = propertyName;
		
		StringBuffer multipleClasses = new StringBuffer();
		for(Class<?> clazz: matchedClasses) {
			if(multipleClasses.length() > 0)
				multipleClasses.append(", ");
			multipleClasses.append(clazz.getName());
		}
			
		params[2] = multipleClasses.toString();
		
		return  I18NUtils.getResource(  "exception.multipleClassForProperty",I18NUtils.CORE_RESOURCES, params);			
		
	}
	
}
