/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model;

public final class AssociationSetting {
	private final MatchType matchType;
	private final Class<?> entityClass;
	private final String pathSuffix;

	public AssociationSetting(Class<?> entityClass) {
		this.entityClass = entityClass;
		this.matchType = MatchType.TYPE;
		this.pathSuffix = null;
	}

	public AssociationSetting(String pathSuffix) {
		this.pathSuffix = pathSuffix;
		this.matchType = MatchType.PATH;
		this.entityClass = null;
	}

	public MatchType getMatchType() {
		return matchType;
	}	

	public boolean isAggregatePart(CallInfo ci) {
		if(matchType == MatchType.TYPE) {
			Object associatedInstance = ci.getOutput() != null ? ((ExtendedDataObject)ci.getOutput()).getInstance() : null;
			return entityClass.isAssignableFrom(associatedInstance.getClass());
		}

		if(matchType == MatchType.PATH && ci.isPathSuffix(this.pathSuffix))
			return true;

		return false;
	}
}
