/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model.view;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

public class HibernateQuery implements Query {
	
	private org.hibernate.Query hibQuery;

	public HibernateQuery(org.hibernate.Query hibQuery) {
		this.hibQuery = hibQuery;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public List getResultList(ViewBranch viewBranch) {
		return hibQuery.list();
	}

	@Override
	public Object getSingleResult() {
		return hibQuery.uniqueResult();
	}

	@Override
	public void setParameter(String name, Object value) {
		hibQuery.setParameter(name, value);
	}

	@Override
	public boolean hasParameter(String name) {
		return (new HashSet<String>(Arrays.asList(hibQuery.getNamedParameters()))).contains(name);
	}

	@Override
	public void setMaxResults(int limit) {
		hibQuery.setMaxResults(limit);
	}

	@Override
	public void setFirstResult(int offset) {
		hibQuery.setFirstResult(offset);
	}

}
