/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.service;

import java.util.Map;

import javax.inject.Inject;

import ws.ladder.model.ExtendedDataObject;
import ws.ladder.model.Type;
import ws.ladder.model.view.Query;
import ws.ladder.util.ClassUtil;

import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;

public class MongoDBPersistenceOrchestrator extends AbstractPersistenceOrchestrator {

	@Inject
	MongoDBFactory mongoDBFactory;

	DB db;   

	protected DB getMongo() {
		db = mongoDBFactory.getMongoDB();
		return db;
	}

	/**
	 * Save the entity in the persistence store
	 * 
	 * @param entity
	 * @return
	 */
	@Override
	public void saveOrUpdate(Object entity) {
		ExtendedDataObject entityDO = (ExtendedDataObject) entity;

		DBCollection coll = db.getCollection(ClassUtil.getBucketName(entityDO.getType().getInstanceClass()));
		coll.save(toDBObject(entityDO));
	}

	/**
	 * Delete the entity from the persistence store
	 * 
	 * @param entity
	 */
	@Override
	public void delete(Object entity) {
		ExtendedDataObject entityDO = (ExtendedDataObject) entity;

		DBCollection coll = db.getCollection(ClassUtil.getBucketName(entityDO.getType().getInstanceClass()));
		coll.remove(toDBObject(entityDO));
	}

	@Override
	public void flush() {

	}

	@Override
	public Object findById(Class<?> persistentClass, Object id) {
		DBCollection coll = db.getCollection(ClassUtil.getBucketName(persistentClass));

		DBObject query = new BasicDBObject();
		query.put("_id", id);
		return coll.find( query);
	}

	@Override
	public Object findByProperty(Type type, Map<String, String> propertyValues) {
		Class<?> persistentClass = type.getInstanceClass();
		DBCollection coll = db.getCollection(ClassUtil.getBucketName(persistentClass));

		// Need to use the query API
		DBObject query = new BasicDBObject();
		for(String property: propertyValues.keySet())
			query.put(property, propertyValues.get(property));

		return coll.find( query);
	}		

	@Override
	public boolean supportsVersionTracking() {
		return false;
	}

	@Override
	public QueryCapability getQueryCapability() {
		return new EmptyQueryCapability();
	}

	@Override
	public boolean canProcessAggregate() {
		return true;
	}	

	private DBObject toDBObject(ExtendedDataObject entityDO) {
		DBObject result = new BasicDBObject();

		// TODO: populate result object

		return result;
	}
	
	@Override
	public Query getQuery(String queryString, boolean isSQL) {
		return null;
	}
}