/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import ws.ladder.model.Property;
import ws.ladder.model.SimpleType;
import ws.ladder.model.Type;
import ws.ladder.util.DFAtoRE.LiteralExpression;
import ws.ladder.util.DFAtoRE.State;

public class AggregatePropertyPaths {
	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());	

	private static Map<Type, Set<String>> typePaths = new ConcurrentHashMap<Type, Set<String>>();   
	
	public static Set<String> enumerate(Type aggregateType) {
		Set<String> paths = typePaths.get(aggregateType);

		if(!typePaths.containsKey(aggregateType)) { 
			DFAtoRE dfaRE = new DFAtoRE(aggregateType);
			Map<DFAtoRE.State, DFAtoRE.Expression> expressions = dfaRE.getRegEx();

			paths  = new HashSet<String>();
			for(State state: expressions.keySet()) {
				Type type = state.getType();
				String prefix = expressions.get(state).toString();
				if(prefix.equals(LiteralExpression.EMPTY_STRING.toString()))
					prefix = "";
				for(Property childProperty: type.getProperties()) {
					if(childProperty != null && !childProperty.isMany() && SimpleType.class.isAssignableFrom(childProperty.getType().getClass())) {
						String path = prefix.concat(childProperty.getName());
						paths.add(path);
					}
				}
			}

			typePaths.put(aggregateType, paths);
		}

		return Collections.unmodifiableSet(paths);
	}
}