/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.engine.spi.CascadeStyle;
import org.hibernate.engine.spi.CascadingAction;
import org.hibernate.engine.spi.SessionFactoryImplementor;
import org.hibernate.mapping.Property;
import org.hibernate.persister.collection.CollectionPersister;
import org.hibernate.type.CollectionType;
import org.hibernate.type.Type;

public class Hibernate4Support implements HibernateSupport {
	
	@Override
	public CollectionPersister getCollectionPersister(SessionFactory sessionFactory, CollectionType collType) {
		return ((SessionFactoryImplementor) sessionFactory).getCollectionPersister( collType.getRole() );
	}

	@Override
	public boolean isCascaded(Configuration configuration, String entityName, String inputPropertyName) {
		
		if(configuration.getClassMapping(entityName) == null)
			return false;

		Property property = configuration.getClassMapping(entityName).getProperty(inputPropertyName);
		CascadeStyle style = property.getCascadeStyle();

		return style.doCascade(CascadingAction.SAVE_UPDATE) 
				|| style.doCascade(CascadingAction.PERSIST);
	}

	@Override
	public Type getEntityType(SessionFactory sessionFactory, String entityName) {
		return sessionFactory.getTypeHelper().entity(entityName);
	}

	@Override
	public Session getCurrentSession(SessionFactory sessionFactory) {
		return sessionFactory.getCurrentSession();
	}		
}
