/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util;

import java.util.Collection;
import java.util.LinkedList;


public class UniqueIdentityArrayList<T> extends IdentityArrayList<T>
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -7633241118173209011L;

	@Override
	public boolean add(T e) {
		if (this.contains(e)) {
			return false;
		}
		else {
			return super.add(e);
		}
	}

	@Override
	public boolean addAll(Collection<? extends T> collection) {
		Collection<T> copy = new LinkedList<T>(collection);
		copy.removeAll(this);
		return super.addAll(copy);
	}

	@Override
	public boolean addAll(int index, Collection<? extends T> collection) {
		Collection<T> copy = new LinkedList<T>(collection);
		copy.removeAll(this);
		return super.addAll(index, copy);
	}

	@Override
	public void add(int index, T element) {
		if (this.contains(element)) {
			return;
		}
		else {
			super.add(index, element);
		}
	}
}